//
//  StencilCanonical.cpp
//  MANITEST
//
//  Created by Alhussein Fawzi on 15.06.15.
//  Copyright (c) 2015 Alhussein Fawzi. All rights reserved.
//

#include "StencilCanonical.h"


std::vector<int> StencilCanonical::getReversedStencilsId(point* coord) {
    // This function returns the CENTER of the different stencils in which coord participates
    int i; std::vector<int> ret;
    point tmp;
    
    for (i=0;i<coord->size();i++) {
        tmp = (*coord); tmp[i] = tmp[i] + 1;
        if (checkOutOfBounds(tmp)) {
            ret.push_back(initializeNode(tmp));
        }
        
        tmp = (*coord); tmp[i] = tmp[i] - 1;
        if (checkOutOfBounds(tmp)) {
            ret.push_back(initializeNode(tmp));
        }
    }
    return ret;
}

void StencilCanonical::genSimplicesId(point* coord, point* y) {
    
    // Clear the simplices
    simplexSetId.clear();
    
    // Check the dimension in which the difference between coord and y occurs
    int i; int y_dim = -1;
    for (i=0; i<(*coord).size(); i++) {
        if ((*coord)[i] != (*y)[i]) {
            y_dim = i;
            break;
        }
    }
    
    assert(y_dim >= 0);
    
    // Populate simplexSet with the recursive function
    genSimplicesRecId(coord, y, y_dim, std::vector<int>(0), 0);
    
    // cout << "size simplex = " << simplexSetId.size() << endl;
}

void StencilCanonical::genSimplicesRecId(point* coord, point *y, int y_dim, std::vector<int> cur_simplex, int cur_dim) {
    
    // Recursive function to populate simplexSet
    if (cur_dim == coord->size()) {
        if (cur_simplex.size() <= 1) {
            // It it's an empty simplex, or a vertex, return
            return;
        }
        simplexSetId.push_back(cur_simplex);
        return;
    }
    if (cur_dim != y_dim) {
        genSimplicesRecId(coord, y, y_dim, cur_simplex, cur_dim+1);
        
        // Add point
        point curPoint = (*coord); curPoint[cur_dim] += 1;
        auto it = parentClass->coords2Id.find(curPoint);
        
        std::vector<int> augmented;
        if (it != parentClass->coords2Id.end()) { // Check that curPoint exists already (TODO record the id here)
            if (parentClass->visited[it->second]) { // Check that it is an 'accepted' point
                augmented = cur_simplex; augmented.push_back(it->second);
                genSimplicesRecId(coord, y, y_dim, augmented, cur_dim+1);
            }
        }
        
        curPoint = (*coord); curPoint[cur_dim] -= 1;
        it = parentClass->coords2Id.find(curPoint);
        
        if (it != parentClass->coords2Id.end()) { // Check that curPoint exists already (TODO record the id here)
            if (parentClass->visited[it->second]) { // Check that it is an 'accepted' point
                augmented = cur_simplex; augmented.push_back(it->second);
                genSimplicesRecId(coord, y, y_dim, augmented, cur_dim+1);
            }
        }
    }
    
    else {
        auto it = parentClass->coords2Id.find(*y);
        std::vector<int> augmented = cur_simplex; augmented.push_back(it->second);
        genSimplicesRecId(coord, y, y_dim, augmented, cur_dim+1);
    }
    
}